@@ -155,6 +155,12 @@ describe("type syntax", function () {
     check("declare function foo(c: (e: Event) => void, b: B): void;");
     check("declare function foo(c: C, d?: Array<D>): void;");
     check("declare class C { x: string }");
+    check("declare class A<X> extends B<X[]> { x: X }");
+    check(
+      "declare class A extends B implements I<string>, J {}",
+      flowParserParseOptions,
+    );
+
     check(
       "declare module M {" +
         eol +
